import contextlib
import os
# import requests
import openai
from .base import Agent
import backoff

class RestAPI(Agent):
    """
    Generic class for creating an Agent based on an URL and API token.
    """

    def __init__(self, name, token="REST_API_KEY", family=None, generations: int = 10):
        # def __init__(self, name, config: AgentConfig=None):

        if family is not None:
            self.family = family
        else:
            self.family = "REST"

        # check for the environment variable
        if token not in os.environ:
            raise ValueError(
                f"Put the {self.family} API token in the {token} environment variable\n \
                e.g.: export {token}='abc123'"
            )
        os.environ["REST_API_KEY"] = os.getenv(token)

        """
        Initializes the Agent class, given a `name`.
        """
        print(f"Loading {self.family} Agent: {name}")
        with contextlib.redirect_stdout(None):
            super().__init__(name=name, generations=generations)

        # determine base uri
        self.uri = self.name + "https://bauzn496g1fv9lv0.us-east-1.aws.endpoints.huggingface.cloud/v1/chat/completions"

        # determine model
        self.model = 'tgi'        

        # add api and base url to client to not conflict with other openai clients in the env
        self.agent = openai.OpenAI(api_key=os.getenv(token), base_url=self.uri)

    @backoff.on_exception(
        backoff.fibo,
        (
            openai.BadRequestError,
            openai.AuthenticationError,
            openai.PermissionDeniedError,
            openai.NotFoundError,
            openai.UnprocessableEntityError,
            openai.RateLimitError,
            openai.InternalServerError,
            openai.APIConnectionError,
        ),
        max_value=70,
    )
    def _call_model(self, prompt):
        response = self.agent.chat.completions.create(
            model=self.model,
            messages=[
                # {"role": "system", "content": "You are a helpful assistant."},
                {"role": "user", "content": prompt},
            ],
            stream = True,
            max_tokens=self.max_tokens,
            temperature=self.temperature,
            presence_penalty=self.presence_penalty,
        )
        return response.choices[0].message.content.strip()

default_class = "RestAPI"

# headers = {
# 	"Accept" : "application/json",
# 	"Authorization": f"""Bearer {REST_API_TOKEN}""",
# 	"Content-Type": "application/json" 
# }

# def query(payload):
# 	response = requests.post(API_URL, headers=headers, json=payload)
# 	return response.json()

# output = query({
# 	"inputs": "<s>[INST] <<SYS>> You are a helpful assistant. You keep your answers short. <</SYS>> What is your favourite condiment? [/INST] My favorite condiment is ketchup. It is versatile, tasty, and goes well with a variety of foods. </s><s>[INST] And what do you think about it? [/INST]",
# 	"parameters": {
# 		"max_new_tokens": 150
# 	}
# })

# from garak.generators.rasa import RasaRestGenerator

# class RestAPI(RasaRestGenerator):
#     """
#     Generic class for creating an Agent based on OpenAI style base URL, name, and API token.
#     """

#     def __init__(self, name, token="REST_API_TOKEN", family=None, generations: int = 10):
#         # def __init__(self, name, config: AgentConfig=None):

#         if family is not None:
#             self.family = family
#         else:
#             self.family = "REST"

#         # check for the environment variable
#         if token not in os.environ:
#             raise ValueError(
#                 f"Put the {self.family} API token in the {token} environment variable\n \
#                 e.g.: export {token}='abc123'"
#             )
#         os.environ["RASA_API_KEY"] = os.getenv(token)

#         """
#         Initializes the Agent class, given a `name`.
#         """
#         print(f"Loading {self.family} Agent: {name}")
#         with contextlib.redirect_stdout(None):
#             super().__init__(uri=name, generations=generations)

#         config = _get_default_agent_config(family="", name=self.name)
#         for field in [
#             "endpoint",
#             "max_tokens",
#             "presence_penalty",
#             "temperature",
#             "top_k",
#             "seed",
#             "presence_penalty",
#             "supports_multiple_generations",
#         ]:
#             if hasattr(config, field):
#                 setattr(self, field, getattr(config, field))
